/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.operator;

import org.apache.commons.lang3.RandomStringUtils;
import org.springframework.stereotype.Component;

import io.fabric8.kubernetes.api.model.ObjectMeta;
import io.fabric8.kubernetes.api.model.Secret;
import io.fabric8.kubernetes.api.model.SecretBuilder;
import lombok.RequiredArgsConstructor;

@RequiredArgsConstructor
@Component
class OzgCloudElasticsearchSecretHelper {

	static final String SECRET_TYPE = "Opaque";

	static final String CREDENTIAL_SECRET_ADDRESS_FIELD = "address";
	static final String CREDENTIAL_SECRET_INDEX_FIELD = "index";
	static final String CREDENTIAL_SECRET_PASSWORD_FIELD = "password";
	static final String CREDENTIAL_SECRET_USERNAME_FIELD = "username";

	static final String CERTIFICATE_SECRET_DATA_KEY = "ca.crt";

	static final int PASSWORD_LENGTH = 15;

	private final OzgCloudElasticsearchProperties properties;

	public Secret buildCredentialSecret(String namespace, String name) {
		return new SecretBuilder()
				.withType(SECRET_TYPE)
				.withMetadata(createMetaData(name, namespace))
				.addToStringData(CREDENTIAL_SECRET_ADDRESS_FIELD, buildSecretAddress())
				.addToStringData(CREDENTIAL_SECRET_INDEX_FIELD, namespace)
				.addToStringData(CREDENTIAL_SECRET_PASSWORD_FIELD, generatePassword())
				.addToStringData(CREDENTIAL_SECRET_USERNAME_FIELD, namespace)
				.build();
	}

	private String buildSecretAddress() {
		return String.format("%s:%s", properties.getServer().getHost(), properties.getServer().getPort());
	}

	private String generatePassword() {
		return RandomStringUtils.randomAlphabetic(PASSWORD_LENGTH);
	}

	public Secret buildCertificateSecret(String namespace, String data) {
		return new SecretBuilder()
				.withType(SECRET_TYPE)
				.withMetadata(createMetaData(properties.getCertificateSecretName(), namespace))
				.addToData(CERTIFICATE_SECRET_DATA_KEY, data)
				.build();
	}

	private ObjectMeta createMetaData(String name, String namespace) {
		var metadata = new ObjectMeta();
		metadata.setName(name);
		metadata.setNamespace(namespace);

		return metadata;
	}
}
