/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.operator;

import static org.assertj.core.api.Assertions.*;

import java.net.HttpURLConnection;

import org.junit.Rule;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;

import io.fabric8.kubernetes.api.model.PodBuilder;
import io.fabric8.kubernetes.api.model.PodList;
import io.fabric8.kubernetes.api.model.PodListBuilder;
import io.fabric8.kubernetes.client.NamespacedKubernetesClient;
import io.fabric8.kubernetes.client.server.mock.KubernetesServer;

class ElasticsearchReconcilerITCase {

	@Rule
	private final KubernetesServer server = new KubernetesServer();
	private NamespacedKubernetesClient client;

	@BeforeEach
	void init() {
		server.before();
		client = server.getClient();
	}

	@DisplayName("Reconcile")
	@Nested
	class TestReconcile {

		private final PodList podList = new PodListBuilder().withItems(
				new PodBuilder().withNewMetadata().withName("pod1").endMetadata()
						.build(),
				new PodBuilder().withNewMetadata().withName("pod2").endMetadata()
						.build())
				.build();

		@Test
		public void shouldGetMockedPods() {
			server.expect().get().withPath("/api/v1/pods").andReturn(HttpURLConnection.HTTP_OK, podList).once();

			var podList = client.pods().inAnyNamespace().list();

			assertThat(podList).isNotNull();
			assertThat(podList.getItems()).hasSize(2);
		}

		@Test
		public void shouldGetMockedPodsInNamespace() {
			var namespace = "default";
			server.expect().get().withPath("/api/v1/namespaces/" + namespace + "/pods")
					.andReturn(HttpURLConnection.HTTP_OK, new PodListBuilder().build())
					.once();

			var podList = client.pods().inNamespace(namespace).list();

			assertThat(podList.getItems()).isEmpty();
		}
	}
}
