/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.operator.common.elasticsearch;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.util.Base64;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import de.ozgcloud.operator.OzgCloudElasticsearchProperties;
import de.ozgcloud.operator.OzgCloudElasticsearchProperties.OzgCloudElasticsearchServerProperties;
import de.ozgcloud.operator.common.kubernetes.KubernetesRemoteService;
import de.ozgcloud.operator.common.kubernetes.SecretTestFactory;
import io.fabric8.kubernetes.api.model.Secret;
import io.fabric8.kubernetes.client.dsl.Resource;

class ElasticsearchClientConfigurationTest {
	
	@Spy
	@InjectMocks
	private ElasticsearchClientConfiguration configuration;
	@Mock
	private OzgCloudElasticsearchProperties properties;
	@Mock
	private KubernetesRemoteService kubernetesRemoteService;
	
	@DisplayName("Create elasticsearch client")
	@Nested
	class TestCreateElasticsearchClient {
		
		private static final String SECRET_DATA_KEY = "dsefsfef";
		private static final String SECRET_DATA_VALUE = "testPassword";
		private static final String SECRET_DATA_ENCODED_VALUE = encodeStringBase64(SECRET_DATA_VALUE);
		private static final Secret SECRET = SecretTestFactory.createBuilder().addToData(SECRET_DATA_KEY, SECRET_DATA_ENCODED_VALUE).build(); 
		
		@Mock
		private Resource<Secret> secretResource;
		@Mock
		private OzgCloudElasticsearchServerProperties serverProperties;
		
		@BeforeEach
		void mock() {
			when(properties.getServer()).thenReturn(serverProperties);
			when(serverProperties.getSecretDataKey()).thenReturn(SECRET_DATA_KEY);
			when(kubernetesRemoteService.getSecretResource(any(), any())).thenReturn(secretResource);
			when(secretResource.get()).thenReturn(SECRET);
		}
		
		@Test
		void shouldReturnPasssowrd() {
			var password = configuration.getPassword();
			
			assertThat(password).isEqualTo(SECRET_DATA_VALUE);
		}
		
		private static String encodeStringBase64(String string) {
			return Base64.getEncoder().encodeToString(string.getBytes());
		}
	}
}