/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.operator.common.kubernetes;

import static org.assertj.core.api.Assertions.*;

import org.junit.Rule;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;

import io.fabric8.kubernetes.api.model.Secret;
import io.fabric8.kubernetes.client.KubernetesClient;
import io.fabric8.kubernetes.client.extension.ResourceAdapter;
import io.fabric8.kubernetes.client.server.mock.KubernetesServer;

class KubernetesRemoteServiceTest {

	@Rule
	private KubernetesServer server;
	private KubernetesClient client;
	private KubernetesRemoteService service;

	@BeforeEach
	void init() {
		server = new KubernetesServer(true, true);
		server.before();
		client = server.getClient();
		service = new KubernetesRemoteService(client);
	}

	@DisplayName("Get secret")
	@Nested
	class TestGetSecret {
		
		private Secret secret = SecretTestFactory.create();
		
		@Test
		void shouldReturnExistingResourceIfExists() {
			createSecret();
			
			var secret = getSecret();

			assertThat(secret).isNotNull().isEqualTo(secret);
		}
		
		private void createSecret() {
			var secretResource = client.secrets().withName(SecretTestFactory.NAME);
			var adapter = new ResourceAdapter<>(secretResource);
			adapter.create(secret);
		}

		@Test
		void shouldReturnNullNOTExists() {
			var secret = getSecret();

			assertThat(secret).isNull();
		}
		
		private Secret getSecret() {
			return service.getSecretResource(NamespaceTestFactory.NAMESPACE, SecretTestFactory.NAME).get();
		}
	}
}